package dev.w9r.config

import org.springframework.boot.actuate.autoconfigure.security.servlet.EndpointRequest
import org.springframework.context.ApplicationContext
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.core.annotation.Order
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.config.annotation.web.invoke
import org.springframework.security.core.userdetails.User
import org.springframework.security.core.userdetails.UserDetailsService
import org.springframework.security.provisioning.InMemoryUserDetailsManager
import org.springframework.security.web.SecurityFilterChain
import org.springframework.security.web.header.writers.XXssProtectionHeaderWriter

@Configuration(proxyBeanMethods = false)
@EnableWebSecurity
open class SecurityConfiguration {
    @Bean
    @Order(-100)
    @Throws(Exception::class)
    open fun securityFilterChain(http: HttpSecurity): SecurityFilterChain {
        http.invoke {
            headers {
                contentSecurityPolicy {
                    policyDirectives = "default-src 'self'"
                }
                cacheControl {
                    disable()
                }
                xssProtection {
                    headerValue = XXssProtectionHeaderWriter.HeaderValue.ENABLED
                }
                httpStrictTransportSecurity {
                    includeSubDomains = false
                    preload = true
                    maxAgeInSeconds = 31536000
                }
            }
            securityMatcher(
                EndpointRequest.toAnyEndpoint()
            )
            csrf {
                ignoringRequestMatchers("/actuator/**")
            }
            authorizeHttpRequests {
                authorize("/actuator/health", permitAll)
                authorize("/actuator/health/**", permitAll)
                authorize("/actuator/info", permitAll)
                authorize("/actuator/info/**", permitAll)
                authorize(anyRequest, hasRole("ENDPOINT_ADMIN"))
            }
            httpBasic {}
        }
        return http.build()
    }

    @Bean
    open fun userDetailsService(): UserDetailsService {
        val userDetails = User.withDefaultPasswordEncoder()
            .username("actuator")
            .password("ac7u80r!")
            .roles("ENDPOINT_ADMIN")
            .build()

        return InMemoryUserDetailsManager(userDetails)
    }

    @Bean
    open fun stateChangeEndpoint(context: ApplicationContext): StateChangeEndpoint {
        return StateChangeEndpoint(context)
    }
}
